@@ -2,24 +2,30 @@
 
 
 import os
-import fpdiff, logging, shutil
+import logging
+import shutil
+from texttestlib.default import fpdiff
 from texttestlib import plugins
 from optparse import OptionParser
-from StringIO import StringIO
+from io import StringIO
+
 
 class Filtering(plugins.TestState):
     def __init__(self, name, **kw):
-        plugins.TestState.__init__(self, name, briefText="", **kw)    
-        
+        plugins.TestState.__init__(self, name, briefText="", **kw)
+
 # Generic base class for filtering standard and temporary files
+
+
 class FilterAction(plugins.Action):
     def __init__(self, useFilteringStates=False):
         self.diag = logging.getLogger("Filter Actions")
         self.useFilteringStates = useFilteringStates
+
     def __call__(self, test):
         if self.useFilteringStates:
             self.changeToFilteringState(test)
-            
+
         for fileName, postfix in self.filesToFilter(test):
             self.diag.info("Considering for filtering : " + fileName)
             stem = self.getStem(fileName)
@@ -29,19 +35,20 @@ class FilterAction(plugins.Action):
     def getStem(self, fileName):
         return os.path.basename(fileName).split(".")[0]
 
-    def changeToFilteringState(self, *args): # pragma: no cover - documentation only
+    def changeToFilteringState(self, *args):  # pragma: no cover - documentation only
         pass
 
     def performAllFilterings(self, test, stem, fileName, newFileName):
         currFileName = fileName
         filters = self.makeAllFilters(test, stem, test.app)
-        for fileFilter, extraPostfix in filters:
-            writeFileName = newFileName + extraPostfix
-            self.diag.info("Applying " + fileFilter.__class__.__name__ + " to make\n" + writeFileName + " from\n " + currFileName) 
+        for fileFilter in filters:
+            writeFileName = newFileName + "." + fileFilter.postfix
+            self.diag.info("Applying " + fileFilter.__class__.__name__ +
+                           " to make\n" + writeFileName + " from\n " + currFileName)
             if os.path.isfile(writeFileName):
                 self.diag.info("Removing previous file at " + writeFileName)
                 os.remove(writeFileName)
-            currFile = open(currFileName, "rU") # use universal newlines to simplify
+            currFile = open(currFileName, errors="ignore")
             writeFile = plugins.openForWrite(writeFileName)
             fileFilter.filterFile(currFile, writeFile)
             writeFile.close()
@@ -49,17 +56,20 @@ class FilterAction(plugins.Action):
         if len(filters) > 0 and currFileName != newFileName:
             shutil.move(currFileName, newFileName)
 
-    def getFilteredText(self, test, fileName, app):
+    def getAllFilters(self, test, fileName, app):
         stem = self.getStem(fileName)
-        filters = self.makeAllFilters(test, stem, app)
-        inFile = open(fileName)
+        return self.makeAllFilters(test, stem, app)
+
+    def getFilteredText(self, test, fileName, app):
+        filters = self.getAllFilters(test, fileName, app)
+        inFile = open(fileName, errors="ignore")
         if len(filters) == 0:
             try:
                 return inFile.read()
             finally:
                 inFile.close()
-        for fileFilter, _ in filters:
-            self.diag.info("Applying " + fileFilter.__class__.__name__ + " to " + fileName) 
+        for fileFilter in filters:
+            self.diag.info("Applying " + fileFilter.__class__.__name__ + " to " + fileName)
             outFile = StringIO()
             fileFilter.filterFile(inFile, outFile)
             inFile.close()
@@ -68,35 +78,34 @@ class FilterAction(plugins.Action):
         value = outFile.getvalue()
         outFile.close()
         return value
-        
+
     def makeAllFilters(self, test, stem, app):
         filters = self._makeAllFilters(test, stem, app)
         if len(filters) == 0 and self.changedOs(app):
-            return  [ (RunDependentTextFilter([], ""), "") ]
+            return [RunDependentTextFilter([], "")]
         else:
             return filters
 
-    def _makeAllFilters(self, test, stem, app):                    
+    def _makeAllFilters(self, test, stem, app):
         filters = []
         configObj = test
-        if test.app is not app: # happens when testing filtering in the static GUI
+        if test.app is not app:  # happens when testing filtering in the static GUI
             configObj = app
-        runDepTexts = configObj.getCompositeConfigValue("run_dependent_text", stem)
-        if runDepTexts:
-            filters.append((RunDependentTextFilter(runDepTexts, test.getRelPath()), ".normal"))
 
-        unorderedTexts = configObj.getCompositeConfigValue("unordered_text", stem)
-        if unorderedTexts:
-            filters.append((UnorderedTextFilter(unorderedTexts, test.getRelPath()), ".sorted"))
+        for filterClass in [RunDependentTextFilter, UnorderedTextFilter]:
+            texts = configObj.getCompositeConfigValue(filterClass.configKey, stem)
+            if texts:
+                filters.append(filterClass(texts, test.getRelPath()))
+
         return filters
-            
+
     def changedOs(self, app):
         homeOs = app.getConfigValue("home_operating_system")
         return homeOs != "any" and os.name != homeOs
 
     def constantPostfix(self, files, postfix):
-        return [ (file, postfix) for file in files ]
-        
+        return [(file, postfix) for file in files]
+
 
 class FilterOriginal(FilterAction):
     def filesToFilter(self, test):
@@ -110,23 +119,25 @@ class FilterOriginal(FilterAction):
         test.changeState(Filtering("initial_filter", executionHosts=execMachines,
                                    freeText=freeText, lifecycleChange="start initial filtering"))
 
-    
-class FilterTemporary(FilterAction):
-    def filesToFilter(self, test):
-        return self.constantPostfix(test.listTmpFiles(), "cmp")
 
+class FilterOnTempFile(FilterAction):
     def _makeAllFilters(self, test, stem, app):
         filters = FilterAction._makeAllFilters(self, test, stem, app)
-        floatTolerance = app.getCompositeConfigValue("floating_point_tolerance", stem)
-        relTolerance = app.getCompositeConfigValue("relative_float_tolerance", stem)
+        floatTolerance = test.getCompositeConfigValue("floating_point_tolerance", stem)
+        relTolerance = test.getCompositeConfigValue("relative_float_tolerance", stem)
         if floatTolerance or relTolerance:
             origFile = test.makeTmpFileName(stem + "." + app.name + "origcmp", forFramework=1)
             if not os.path.isfile(origFile):
                 origFile = test.getFileName(stem)
             if origFile and os.path.isfile(origFile):
-                filters.append((FloatingPointFilter(origFile, floatTolerance, relTolerance), ".fpdiff"))
+                filters.append(FloatingPointFilter(origFile, floatTolerance, relTolerance))
         return filters
 
+
+class FilterTemporary(FilterOnTempFile):
+    def filesToFilter(self, test):
+        return self.constantPostfix(test.listTmpFiles(), "cmp")
+
     def changeToFilteringState(self, test):
         # Notifications of current status are only useful when doing normal filtering in the GUI
         execMachines = test.state.executionHosts
@@ -146,15 +157,15 @@ class FilterOriginalForScript(FilterOriginal):
 class FilterErrorText(FilterAction):
     def _makeAllFilters(self, test, stem, app):
         texts = app.getConfigValue("suppress_stderr_text")
-        return [ (RunDependentTextFilter(texts), "") ]
-    
+        return [RunDependentTextFilter(texts)]
 
-class FilterProgressRecompute(FilterAction):
+
+class FilterProgressRecompute(FilterOnTempFile):
     def filesToFilter(self, test):
         return self.constantPostfix(test.listTmpFiles(), "partcmp")
 
 
-class FilterResultRecompute(FilterAction):
+class FilterResultRecompute(FilterOnTempFile):
     def filesToFilter(self, test):
         result = []
         for fileComp in test.state.allResults:
@@ -165,7 +176,10 @@ class FilterResultRecompute(FilterAction):
                 result.append((fileComp.tmpFile, "cmp"))
         return result
 
+
 class FloatingPointFilter:
+    postfix = "fpdiff"
+
     def __init__(self, origFileName, tolerance, relative):
         self.origFileName = origFileName
         self.tolerance, self.relative = None, None
@@ -175,16 +189,19 @@ class FloatingPointFilter:
             self.relative = relative
 
     def filterFile(self, inFile, writeFile):
-        fromlines = open(self.origFileName, "rU").readlines()
+        fromlines = open(self.origFileName, errors="ignore").readlines()
         tolines = inFile.readlines()
         fpdiff.fpfilter(fromlines, tolines, writeFile, self.tolerance, self.relative)
 
 
 class RunDependentTextFilter(plugins.Observable):
+    configKey = "run_dependent_text"
+    postfix = "normal"
+
     def __init__(self, filterTexts, testId=""):
         plugins.Observable.__init__(self)
         self.diag = logging.getLogger("Run Dependent Text")
-        self.lineFilters = [ LineFilter(text, testId, self.diag) for text in filterTexts ]
+        self.lineFilters = [LineFilter(text, testId, self.diag) for text in filterTexts]
 
     def findRelevantFilters(self, file):
         relevantFilters, sectionFilters = [], []
@@ -206,7 +223,7 @@ class RunDependentTextFilter(plugins.Observable):
                         orderedRelevantFilters.append((lineFilter, lastLine))
             return orderedRelevantFilters
         else:
-            return [ (f, None) for f in relevantFilters ]
+            return [(f, None) for f in relevantFilters]
 
     def getLastLine(self, filters, lineFilter):
         for f, lastLine in reversed(filters):
@@ -223,7 +240,7 @@ class RunDependentTextFilter(plugins.Observable):
                     relevantFilters.append((sectionFilter, lineNumber))
             for sectionFilter in sectionFilters:
                 if sectionFilter not in matchedFirst and \
-                    sectionFilter.trigger.matches(line, lineNumber) and not sectionFilter.untrigger.matches(line, lineNumber):
+                        sectionFilter.trigger.matches(line, lineNumber) and not sectionFilter.untrigger.matches(line, lineNumber):
                     matchedFirst.append(sectionFilter)
         for sectionFilter in sectionFilters:
             sectionFilter.trigger.reset()
@@ -258,8 +275,9 @@ class RunDependentTextFilter(plugins.Observable):
         filteredLine = line
         linesToRemove = 0
         filtersToRemove = []
+        alreadyFilteredAway = False
         for lineFilter, lastRelevantLine in lineFilters:
-            changed, currFilteredLine, removeCount = lineFilter.applyTo(line, lineNumber)
+            changed, currFilteredLine, removeCount = lineFilter.applyTo(line, lineNumber, alreadyFilteredAway)
             if lastRelevantLine is not None and lineNumber >= lastRelevantLine:
                 filtersToRemove.append((lineFilter, lastRelevantLine))
 
@@ -270,13 +288,18 @@ class RunDependentTextFilter(plugins.Observable):
                     line = currFilteredLine
                 if filteredLine:
                     filteredLine = currFilteredLine
+                if currFilteredLine is None:
+                    alreadyFilteredAway = True
         for lineFilter, lastRelevantLine in filtersToRemove:
             lineFilters.remove((lineFilter, lastRelevantLine))
-                    
+
         return appliedLineFilter, filteredLine, linesToRemove
 
 
 class UnorderedTextFilter(RunDependentTextFilter):
+    configKey = "unordered_text"
+    postfix = "sorted"
+
     def filterFile(self, file, newFile):
         unorderedLines = {}
         RunDependentTextFilter.filterFile(self, file, newFile, unorderedLines)
@@ -292,53 +315,62 @@ class UnorderedTextFilter(RunDependentTextFilter):
             for line in unordered:
                 newFile.write(line)
             newFile.write("\n")
-  
+
 
 class LineNumberTrigger:
     def __init__(self, lineNumber):
         self.lineNumber = lineNumber
+
     def __repr__(self):
         return "Line number trigger for line " + str(self.lineNumber)
+
     def matches(self, lineArg, lineNumber):
         return lineNumber == self.lineNumber
+
     def replace(self, lineArg, newText):
         return newText
+
     def reset(self):
         pass
 
+
 class MatchNumberTrigger(plugins.TextTrigger):
     def __init__(self, text, matchNumber):
         self.matchNumber = matchNumber
         self.matchCounter = 0
         plugins.TextTrigger.__init__(self, text)
-        
+
     def __repr__(self):
         return "Match number trigger for the " + str(self.matchNumber) + ":th match"
-        
+
     def matches(self, line, *args):
         if plugins.TextTrigger.matches(self, line):
             self.matchCounter += 1
             return self.matchNumber == self.matchCounter
         return False
-    
+
     def reset(self):
         self.matchCounter = 0
 
+
 def getWriteDirRegexp(testId):
+    testId = testId.replace("\\", "/")
     for char in "+^":
         testId = testId.replace(char, "\\" + char)
     # Some stuff, a date, and the testId (ignore the appId as we don't know when or where)
     # Doesn't handle paths with spaces, which seems hard, but does hardcode the default location of $HOME on Windows...
-    posixVersion = '([A-Za-z]:/Documents and Settings)?[^ "=]*/[^ "=]*[0-3][0-9][A-Za-z][a-z][a-z][0-9]{6}[^ "=]*/' + testId
-    return posixVersion.replace("/", "[/\\\\]")
+    posixVersion = '([A-Za-z]:/Documents and Settings)?[^ \'"=]*/[^ "=]*[0-3][0-9][A-Za-z][a-z][a-z][0-9]{6}[^ "=]*/' + testId
+    return posixVersion.replace("/", "[/\\\\]+")
+
 
 class LineFilter:
-    dividers = [ "{->}", "{[->]}", "{[->}", "{->]}" ]
+    dividers = ["{->}", "{[->]}", "{[->}", "{->]}"]
     # All syntax that affects how a match is found
-    matcherStrings = [ "{LINE ", "{INTERNAL ", "{MATCH " ]
+    matcherStrings = ["{LINE ", "{INTERNAL ", "{MATCH "]
     # All syntax that affects what is done when a match is found
-    matchModifierStrings = [ "{WORD ", "{REPLACE ", "{LINES ", "{PREVLINES " ]
-    internalExpressions = { "writedir" : getWriteDirRegexp }
+    matchModifierStrings = ["{WORD ", "{REPLACE ", "{LINES ", "{PREVLINES "]
+    internalExpressions = {"writedir": getWriteDirRegexp}
+
     def __init__(self, text, testId, diag):
         self.originalText = text
         self.testId = testId
@@ -351,14 +383,17 @@ class LineFilter:
         self.wordNumber = None
         self.replaceText = None
         self.divider = None
-        self.removeWordsAfter = 0
+        self.removeWordsAfter = False
         self.parseOriginalText()
         self.diag.info("Created trigger : " + repr(self.trigger))
-        
+
+    def makeNew(self, newText):
+        return LineFilter(newText, self.testId, self.diag)
+
     def getInternalExpression(self, parameter):
         method = self.internalExpressions.get(parameter)
         return method(self.testId)
-    
+
     def makeRegexTrigger(self, parameter):
         expression = self.getInternalExpression(parameter)
         return plugins.TextTrigger(expression)
@@ -373,7 +408,7 @@ class LineFilter:
                 self.untrigger = self.parseText(afterText)
                 return
         self.trigger = self.parseText(self.originalText)
-        
+
     def parseText(self, text):
         for matchModifierString in self.matchModifierStrings:
             linePoint = text.find(matchModifierString)
@@ -399,13 +434,13 @@ class LineFilter:
         parameter = afterText[:endPos]
         afterText = afterText[endPos + 1:]
         return beforeText, afterText, parameter
-    
+
     def readMatchModifier(self, matchModifierString, parameter):
         if matchModifierString == "{REPLACE ":
             self.replaceText = parameter
         elif matchModifierString == "{WORD ":
             if parameter.endswith("+"):
-                self.removeWordsAfter = 1
+                self.removeWordsAfter = True
                 self.wordNumber = int(parameter[:-1])
             else:
                 self.wordNumber = int(parameter)
@@ -420,27 +455,32 @@ class LineFilter:
     def createTrigger(self, matcherString, text, parameter):
         if matcherString == "{LINE ":
             return LineNumberTrigger(int(parameter))
-        elif matcherString == "{INTERNAL " and self.internalExpressions.has_key(parameter):
+        elif matcherString == "{INTERNAL " and parameter in self.internalExpressions:
             return self.makeRegexTrigger(parameter)
-        elif matcherString ==  "{MATCH ":
+        elif matcherString == "{MATCH ":
             return MatchNumberTrigger(text, int(parameter))
         else:
             return plugins.TextTrigger(text)
 
-    def applyTo(self, line, lineNumber=0):
+    def isMultiLine(self):
+        return self.linesToRemove > 1 or self.prevLinesToRemove > 0 or self.untrigger is not None
+
+    def applyTo(self, line, lineNumber=0, alreadyFilteredAway=False):
         if self.autoRemove:
             return self.applyAutoRemove(line)
+        elif alreadyFilteredAway and not self.isMultiLine():
+            return False, None, 0
 
         if self.trigger.matches(line, lineNumber):
             self.diag.info(repr(self.trigger) + " matched " + line.rstrip())
             return self.applyMatchingTrigger(line)
         else:
             return False, line, 0
-        
+
     def applyAutoRemove(self, line):
         if self.untrigger:
             if self.untrigger.matches(line.rstrip()):
-                self.diag.info(repr(self.untrigger) + " (end) matched " + line.rstrip()) 
+                self.diag.info(repr(self.untrigger) + " (end) matched " + line.rstrip())
                 self.autoRemove = 0
                 if self.divider.endswith("]}"):
                     return True, None, 0
@@ -457,7 +497,7 @@ class LineFilter:
         if self.linesToRemove:
             self.autoRemove = self.linesToRemove - 1
         return True, self.filterWords(line, self.trigger), self.prevLinesToRemove
-            
+
     def filterWords(self, line, trigger=None):
         if self.wordNumber != None:
             stripped = line.rstrip()
@@ -476,8 +516,8 @@ class LineFilter:
                         words[realNumber] = self.replaceText
                     else:
                         del words[realNumber]
-            if realNumber == -1 or realNumber >= len(words) - 1: # Trim trailing spaces for words at end or beyond
-                postfix = "\n" 
+            if realNumber == -1 or realNumber >= len(words) - 1:  # Trim trailing spaces for words at end or beyond
+                postfix = "\n"
             return " ".join(words).rstrip() + postfix
         elif trigger and self.replaceText != None:
             return trigger.replace(line.rstrip("\n"), self.replaceText) + "\n"
@@ -492,7 +532,7 @@ class LineFilter:
                     return realWordNumber
                 wordNumber += 1
         return len(words) + 1
-    
+
     def findRealWordNumberBackwards(self, words):
         wordNumber = -1
         for index in range(len(words)):
